# Copyright (C) 2006-2008 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from LuciZopePerm import havePermCreateCluster
from conga_constants import CLUNAME, CLUSTER, \
	CLUSTER_ADD, CLUSTER_CONFIG, CLUSTERLIST, CLUSTERS, \
	FDOM, FDOM_ADD, FDOM_CONFIG, FDOMS, \
	FENCEDEV, FENCEDEV_ADD, FENCEDEV_CONFIG, FENCEDEVS, \
	NODE, NODE_ADD, NODE_CONFIG, NODE_GRID, NODE_LIST, NODES, PAGETYPE, \
	RESOURCE, RESOURCE_ADD, RESOURCE_CONFIG, RESOURCES, \
	SERVICE, SERVICE_ADD, SERVICE_CONFIG, SERVICE_LIST, SERVICES, \
	VM_ADD, VM_CONFIG

# Policy for showing the cluster chooser menu:
# 1) If there are no clusters in the ManagedClusterSystems
# folder, then only the admin user may see this menu, and
# the configure option should not be displayed.
# 2)If there are clusters in the ManagedClusterSystems,
# then only display chooser if the current user has
# permissions on at least one. If the user is admin, show ALL clusters

def createCluChooser(self, request, systems):
	from cluster_adapters import validatePost
	dummynode = {}

	if request.REQUEST_METHOD == 'POST':
		ret = validatePost(self, request)
		try:
			request.SESSION.set('checkRet', ret[1])
		except:
			request.SESSION.set('checkRet', {})
	else:
		try:
			request.SESSION.set('checkRet', {})
		except:
			pass

	# First, see if a cluster is chosen, then
	# check that the current user can access that system
	cname = None
	try:
		cname = request[CLUNAME]
	except:
		cname = ''

	try:
		url = request['URL']
	except:
		url = '/luci/cluster/index_html'

	try:
		pagetype = request[PAGETYPE]
	except:
		pagetype = '3'

	cldata = {}
	cldata['Title'] = 'Cluster List'
	cldata['cfg_type'] = 'clusters'
	cldata['absolute_url'] = '%s?pagetype=%s' % (url, CLUSTERLIST)
	cldata['Description'] = 'Clusters available for configuration'
	if pagetype == CLUSTERLIST:
		cldata['currentItem'] = True
	else:
		cldata['currentItem'] = False

	UserHasPerms = havePermCreateCluster(self)
	if UserHasPerms:
		cladd = {}
		cladd['Title'] = 'Create a New Cluster'
		cladd['cfg_type'] = 'clusteradd'
		cladd['absolute_url'] = '%s?pagetype=%s' % (url, CLUSTER_ADD)
		cladd['Description'] = 'Create a Cluster'
		if pagetype == CLUSTER_ADD:
			cladd['currentItem'] = True
		else:
			cladd['currentItem'] = False

	clcfg = {}
	clcfg['Title'] = 'Configure'
	clcfg['cfg_type'] = 'clustercfg'
	clcfg['absolute_url'] = '%s?pagetype=%s' % (url, CLUSTERS)
	clcfg['Description'] = 'Configure a cluster'
	if pagetype == CLUSTERS:
		clcfg['currentItem'] = True
	else:
		clcfg['currentItem'] = False

	#test...
	#clcfg['show_children'] = True
	#Add all cluster type pages here:
	if pagetype == CLUSTER or pagetype == CLUSTER_CONFIG:
		clcfg['show_children'] = True
	else:
		clcfg['show_children'] = False

	#loop through all clusters
	syslist = list()
	for system in systems:
		clsys = {}
		clsys['Title'] = system[0]
		clsys['cfg_type'] = 'cluster'
		clsys['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, CLUSTER, system[0])
		clsys['Description'] = 'Configure this cluster'

		if pagetype == CLUSTER or pagetype == CLUSTER_CONFIG:
			if cname == system[0]:
				clsys['currentItem'] = True
			else:
				clsys['currentItem'] = False
		else:
			clsys['currentItem'] = False
		syslist.append(clsys)

	clcfg['children'] = syslist

	mylist = list()
	mylist.append(cldata)
	if UserHasPerms:
		mylist.append(cladd)
	mylist.append(clcfg)
	dummynode['children'] = mylist

	return dummynode

def createCluConfigTree(self, request, model):
	dummynode = {}

	if not model:
		return {}

	# There should be a positive page type
	try:
		pagetype = request[PAGETYPE]
	except:
		pagetype = '3'

	try:
		url = request['URL']
	except:
		url = '/luci/cluster/index_html'

	# The only way this method can run is if there exists
	# a clustername query var
	cluname = request['clustername']

	nd = {}
	nd['Title'] = 'Nodes'
	nd['cfg_type'] = 'nodes'
	nd['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, NODES, cluname)
	nd['Description'] = 'Node configuration for this cluster'
	if pagetype == NODES or pagetype == NODE_GRID or pagetype == NODE_LIST or pagetype == NODE_CONFIG or pagetype == NODE_ADD or pagetype == NODE:
		nd['show_children'] = True
	else:
		nd['show_children'] = False
	if pagetype == '0':
		nd['show_children'] = False

	if pagetype == NODES:
		nd['currentItem'] = True
	else:
		nd['currentItem'] = False


	ndadd = {}
	ndadd['Title'] = 'Add a Node'
	ndadd['cfg_type'] = 'nodeadd'
	ndadd['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, NODE_ADD, cluname)
	ndadd['Description'] = 'Add a node to this cluster'
	if pagetype == NODE_ADD:
		ndadd['currentItem'] = True
	else:
		ndadd['currentItem'] = False

	ndcfg = {}
	ndcfg['Title'] = 'Configure'
	ndcfg['cfg_type'] = 'nodecfg'
	ndcfg['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, NODE_CONFIG, cluname)
	ndcfg['Description'] = 'Configure cluster nodes'
	if pagetype == NODE_CONFIG or pagetype == NODE or pagetype == NODES or pagetype == NODE_LIST or pagetype == NODE_GRID or pagetype == NODE_ADD:
		ndcfg['show_children'] = True
	else:
		ndcfg['show_children'] = False
	if pagetype == NODE_CONFIG:
		ndcfg['currentItem'] = True
	else:
		ndcfg['currentItem'] = False

	nodes = model.getNodes()
	nodenames = list()
	for node in nodes:
		nodenames.append(node.getName())

	cfgablenodes = list()
	for nodename in nodenames:
		cfg = {}
		cfg['Title'] = nodename
		cfg['cfg_type'] = 'node'
		cfg['absolute_url'] = '%s?pagetype=%s&nodename=%s&clustername=%s' % (url, NODE, nodename, cluname)
		cfg['Description'] = 'Configure this cluster node'
		if pagetype == NODE:
			try:
				nname = request['nodename']
			except KeyError, e:
				nname = ''
			if nodename == nname:
				cfg['currentItem'] = True
			else:
				cfg['currentItem'] = False
		else:
			cfg['currentItem'] = False

		cfgablenodes.append(cfg)

	#Now add nodename structs as children of the config element
	ndcfg['children'] = cfgablenodes

	ndkids = list()
	ndkids.append(ndadd)
	ndkids.append(ndcfg)

	nd['children'] = ndkids

	##################################################################
	sv = {}
	sv['Title'] = 'Services'
	sv['cfg_type'] = 'services'
	sv['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, SERVICES, cluname)
	sv['Description'] = 'Service configuration for this cluster'
	if pagetype == SERVICES or pagetype == SERVICE_CONFIG or pagetype == SERVICE_ADD or pagetype == SERVICE or pagetype == SERVICE_LIST or pagetype == VM_ADD or pagetype == VM_CONFIG:
		sv['show_children'] = True
	else:
		sv['show_children'] = False
	if pagetype == SERVICES or pagetype == SERVICE_LIST:
		sv['currentItem'] = True
	else:
		sv['currentItem'] = False

	svadd = {}
	svadd['Title'] = 'Add a Service'
	svadd['cfg_type'] = 'serviceadd'
	svadd['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, SERVICE_ADD, cluname)
	svadd['Description'] = 'Add a Service to this cluster'
	if pagetype == SERVICE_ADD:
		svadd['currentItem'] = True
	else:
		svadd['currentItem'] = False

	if model.getIsVirtualized() is True:
		vmadd = {}
		vmadd['Title'] = 'Add a Virtual Machine Service'
		vmadd['cfg_type'] = 'vmadd'
		vmadd['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, VM_ADD, cluname)
		vmadd['Description'] = 'Add a Virtual Machine Service to this cluster'
		if pagetype == VM_ADD:
			vmadd['currentItem'] = True
		else:
			vmadd['currentItem'] = False

	svcfg = {}
	svcfg['Title'] = 'Configure a Service'
	svcfg['cfg_type'] = 'servicecfg'
	svcfg['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, SERVICE_CONFIG, cluname)
	svcfg['Description'] = 'Configure a Service for this cluster'
	if pagetype == SERVICE_CONFIG or pagetype == SERVICE or pagetype == VM_CONFIG:
		svcfg['show_children'] = True
	else:
		svcfg['show_children'] = False
	if pagetype == SERVICE_CONFIG or pagetype == VM_CONFIG:
		svcfg['currentItem'] = True
	else:
		svcfg['currentItem'] = False

	services = model.getServices()
	serviceable = list()

	for service in services:
		servicename = service.getName()
		svc = {}
		svc['Title'] = servicename
		svc['cfg_type'] = 'service'
		svc['absolute_url'] = '%s?pagetype=%s&servicename=%s&clustername=%s' % (url, SERVICE, servicename, cluname)
		svc['Description'] = 'Configure this service'
		if pagetype == SERVICE:
			try:
				sname = request['servicename']
			except KeyError, e:
				sname = ''
			if servicename == sname:
				svc['currentItem'] = True
			else:
				svc['currentItem'] = False
		else:
			svc['currentItem'] = False

		serviceable.append(svc)

	vms = model.getVMs()
	for vm in vms:
		name = vm.getName()
		svc = {}
		svc['Title'] = name
		svc['cfg_type'] = 'vm'
		svc['absolute_url'] = '%s?pagetype=%s&servicename=%s&clustername=%s' % (url, VM_CONFIG, name, cluname)
		svc['Description'] = 'Configure this Virtual Machine Service'
		if pagetype == VM_CONFIG:
			try:
				xname = request['servicename']
			except KeyError, e:
				xname = ''
			if name == xname:
				svc['currentItem'] = True
			else:
				svc['currentItem'] = False
		else:
			svc['currentItem'] = False

		serviceable.append(svc)

	svcfg['children'] = serviceable



	kids = list()
	kids.append(svadd)
	if model.getIsVirtualized() is True:
		kids.append(vmadd)
	kids.append(svcfg)
	sv['children'] = kids
#############################################################
	rv = {}
	rv['Title'] = 'Resources'
	rv['cfg_type'] = 'resources'
	rv['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, RESOURCES, cluname)
	rv['Description'] = 'Resource configuration for this cluster'
	if pagetype == RESOURCES or pagetype == RESOURCE_CONFIG or pagetype == RESOURCE_ADD or pagetype == RESOURCE:
		rv['show_children'] = True
	else:
		rv['show_children'] = False
	if pagetype == RESOURCES:
		rv['currentItem'] = True
	else:
		rv['currentItem'] = False

	rvadd = {}
	rvadd['Title'] = 'Add a Resource'
	rvadd['cfg_type'] = 'resourceadd'
	rvadd['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, RESOURCE_ADD, cluname)
	rvadd['Description'] = 'Add a Resource to this cluster'
	if pagetype == RESOURCE_ADD:
		rvadd['currentItem'] = True
	else:
		rvadd['currentItem'] = False

	rvcfg = {}
	rvcfg['Title'] = 'Configure a Resource'
	rvcfg['cfg_type'] = 'resourcecfg'
	rvcfg['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, RESOURCE_CONFIG, cluname)
	rvcfg['Description'] = 'Configure a Resource for this cluster'
	if pagetype == RESOURCE_CONFIG or pagetype == RESOURCE:
		rvcfg['show_children'] = True
	else:
		rvcfg['show_children'] = False
	if pagetype == RESOURCE_CONFIG:
		rvcfg['currentItem'] = True
	else:
		rvcfg['currentItem'] = False

	resources = model.getResources()
	resourceable = list()
	for resource in resources:
		resourcename = resource.getName()
		rvc = {}
		rvc['Title'] = resourcename
		rvc['cfg_type'] = 'resource'
		rvc['absolute_url'] = '%s?pagetype=%s&resourcename=%s&clustername=%s' % (url, RESOURCES, resourcename, cluname)
		rvc['Description'] = 'Configure this resource'
		if pagetype == RESOURCE:
			try:
				rname = request['resourcename']
			except KeyError, e:
				rname = ''
			if resourcename == rname:
				rvc['currentItem'] = True
			else:
				rvc['currentItem'] = False
		else:
			rvc['currentItem'] = False

		resourceable.append(rvc)
	rvcfg['children'] = resourceable



	kids = list()
	kids.append(rvadd)
	kids.append(rvcfg)
	rv['children'] = kids
 ################################################################
	fd = {}
	fd['Title'] = 'Failover Domains'
	fd['cfg_type'] = 'failoverdomains'
	fd['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, FDOMS, cluname)
	fd['Description'] = 'Failover domain configuration for this cluster'
	if pagetype == FDOMS or pagetype == FDOM_CONFIG or pagetype == FDOM_ADD or pagetype == FDOM:
		fd['show_children'] = True
	else:
		fd['show_children'] = False
	if pagetype == FDOMS:
		fd['currentItem'] = True
	else:
		fd['currentItem'] = False

	fdadd = {}
	fdadd['Title'] = 'Add a Failover Domain'
	fdadd['cfg_type'] = 'failoverdomainadd'
	fdadd['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, FDOM_ADD, cluname)
	fdadd['Description'] = 'Add a Failover Domain to this cluster'
	if pagetype == FDOM_ADD:
		fdadd['currentItem'] = True
	else:
		fdadd['currentItem'] = False

	fdcfg = {}
	fdcfg['Title'] = 'Configure a Failover Domain'
	fdcfg['cfg_type'] = 'failoverdomaincfg'
	fdcfg['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, FDOM_CONFIG, cluname)
	fdcfg['Description'] = 'Configure a Failover Domain for this cluster'
	if pagetype == FDOM_CONFIG or pagetype == FDOM:
		fdcfg['show_children'] = True
	else:
		fdcfg['show_children'] = False
	if pagetype == FDOM_CONFIG:
		fdcfg['currentItem'] = True
	else:
		fdcfg['currentItem'] = False

	fdoms = model.getFailoverDomains()
	fdomable = list()
	for fdom in fdoms:
		fdomname = fdom.getName()
		fdc = {}
		fdc['Title'] = fdomname
		fdc['cfg_type'] = 'fdom'
		fdc['absolute_url'] = '%s?pagetype=%s&fdomname=%s&clustername=%s' % (url, FDOM, fdomname, cluname)
		fdc['Description'] = 'Configure this Failover Domain'
		if pagetype == FDOM:
			try:
				fname = request['fdomname']
			except KeyError, e:
				fname = ''
			if fdomname == fname:
				fdc['currentItem'] = True
			else:
				fdc['currentItem'] = False
		else:
			fdc['currentItem'] = False

		fdomable.append(fdc)
	fdcfg['children'] = fdomable



	kids = list()
	kids.append(fdadd)
	kids.append(fdcfg)
	fd['children'] = kids
#############################################################
	fen = {}
	fen['Title'] = 'Shared Fence Devices'
	fen['cfg_type'] = 'fencedevicess'
	fen['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, FENCEDEVS, cluname)
	fen['Description'] = 'Fence Device configuration for this cluster'
	if pagetype == FENCEDEVS or pagetype == FENCEDEV_CONFIG or pagetype == FENCEDEV_ADD or pagetype == FENCEDEV:
		fen['show_children'] = True
	else:
		fen['show_children'] = False
	if pagetype == FENCEDEVS:
		fen['currentItem'] = True
	else:
		fen['currentItem'] = False

	fenadd = {}
	fenadd['Title'] = 'Add a Fence Device'
	fenadd['cfg_type'] = 'fencedeviceadd'
	fenadd['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, FENCEDEV_ADD, cluname)
	fenadd['Description'] = 'Add a Fence Device to this cluster'
	if pagetype == FENCEDEV_ADD:
		fenadd['currentItem'] = True
	else:
		fenadd['currentItem'] = False

	fencfg = {}
	fencfg['Title'] = 'Configure a Fence Device'
	fencfg['cfg_type'] = 'fencedevicecfg'
	fencfg['absolute_url'] = '%s?pagetype=%s&clustername=%s' % (url, FENCEDEV_CONFIG, cluname)
	fencfg['Description'] = 'Configure a Fence Device for this cluster'
	if pagetype == FENCEDEV_CONFIG or pagetype == FENCEDEV:
		fencfg['show_children'] = True
	else:
		fencfg['show_children'] = False
	if pagetype == FENCEDEV_CONFIG:
		fencfg['currentItem'] = True
	else:
		fencfg['currentItem'] = False

	fences = model.getFenceDevices()
	fenceable = list()
	for fence in fences:
		fencename = fence.getName()
		fenc = {}
		fenc['Title'] = fencename
		fenc['cfg_type'] = 'fencedevice'
		fenc['absolute_url'] = '%s?pagetype=%s&fencename=%s&clustername=%s' % (url, FENCEDEV, fencename, cluname)
		fenc['Description'] = 'Configure this Fence Device'
		if pagetype == FENCEDEV:
			try:
				fenname = request['fencename']
			except KeyError, e:
				fenname = ''
			if fencename == fenname:
				fenc['currentItem'] = True
			else:
				fenc['currentItem'] = False
		else:
			fenc['currentItem'] = False

		fenceable.append(fenc)
	fencfg['children'] = fenceable



	kids = list()
	kids.append(fenadd)
	kids.append(fencfg)
	fen['children'] = kids
#############################################################

	mylist = list()
	mylist.append(nd)
	mylist.append(sv)
	mylist.append(rv)
	mylist.append(fd)
	mylist.append(fen)

	dummynode['children'] = mylist

	return dummynode
