# Copyright (C) 2006-2008 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from conga_constants import PLONE_ROOT, \
	STORAGE_FOLDER_PATH, CLUSTER_FOLDER_PATH, LUCI_DEBUG_MODE

from RicciQueries import getClusterConf
from LuciSyslog import get_logger
from HelperFunctions import resolveOSType

from LuciDB import delCluster, clearNodeStatus, delSystem, \
	getClusterNode, getClusters, getStorage, getStorageNode, \
	manageCluster, \
	CLUSTER_NODE_NEED_AUTH

from LuciClusterInfo import getClusterConfNodes

from LuciZopePerm import havePermAddCluster, havePermRemCluster, \
	havePermAddUser, havePermDelUser, havePermEditPerms, \
	havePermRemStorage, havePermAddStorage

from ricci_communicator import RicciCommunicator

# Homebase area page types
HOMEBASE_ADD_USER				= '1'
HOMEBASE_ADD_SYSTEM				= '2'
HOMEBASE_PERMS					= '3'
HOMEBASE_DEL_USER				= '4'
HOMEBASE_DEL_SYSTEM				= '5'
HOMEBASE_ADD_CLUSTER			= '6'
HOMEBASE_ADD_CLUSTER_INITIAL	= '7'
HOMEBASE_AUTH					= '8'

luci_log = get_logger()

def validateDelSystem(self, request):
	errors = list()
	messages = list()

	if request.form.has_key('__SYSTEM'):
		system_names = request.form['__SYSTEM']
		for i in system_names:
			if not i:
				continue
			err_msg = delSystem(self, i)
			if err_msg:
				errors.append(err_msg)
			else:
				messages.append('Removed storage system "%s" successfully' % i)

	if request.form.has_key('__CLUSTER'):
		cluster_names = request.form['__CLUSTER']
		for i in cluster_names:
			if not i:
				continue
			err_msg = delCluster(self, i)
			if err_msg:
				errors.append(err_msg)
			else:
				messages.append('Removed cluster "%s" successfully' % i)

	return (len(errors) < 1, { 'messages': messages, 'errors': errors })

def validateDelUser(self, request):
	errors = list()
	messages = list()

	if not request.form.has_key('deluserId'):
		return (False, { 'errors': [ 'No User ID given' ] })

	try:
		userId = request.form['deluserId'].strip()
		user = self.portal_membership.getMemberById(userId)
		if not user:
			raise Exception, 'user %s does not exist' % userId
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VDU0: %s: %r %s' % (userId, e, str(e)))
		return (False, {'errors': [ 'No such user: "%s"' % userId ] })

	for i in getClusters(self):
		try:
			i[1].manage_delLocalRoles([ userId ])
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('VDU1: %s %s: %r %s' \
					% (userId, i[0], e, str(e)))
			errors.append('Error deleting roles from cluster "%s" for user "%s"' \
				% (i[0], userId))

	for i in getStorage(self):
		try:
			i[1].manage_delLocalRoles([ userId ])
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('VDU2: %s: %r %s' \
					% (userId, i[0], e, str(e)))
			errors.append('Error deleting roles from storage system "%s" for user "%s"' % (i[0], userId))

	try:
		self.acl_users.userFolderDelUsers([ userId ])
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VDU3: %s %r %s' \
				% (userId, e, str(e)))
		errors.append('Unable to delete user "%s"' % userId)
		return (False, {'errors': errors })

	messages.append('User "%s" has been deleted' % userId)
	return (True, { 'errors': errors, 'messages': messages })

def validateAddUser(self, request):
	messages = list()

	if not request.form.has_key('newUserName'):
		return (False, { 'errors': [ 'No user name given' ] })

	if not request.form.has_key('newPassword'):
		return (False, { 'errors': [ 'No password given' ] })

	if not request.form.has_key('newPasswordConfirm'):
		return (False, { 'errors': [ 'You didn\'t confirm the password' ] })

	user = request.form['newUserName'].strip()

	if self.portal_membership.getMemberById(user):
		return (False, { 'errors': [ 'The user "%s" already exists' % user ]})

	passwd = request.form['newPassword']
	pwconfirm = request.form['newPasswordConfirm']

	if len(passwd) < 6:
		return (False, { 'errors': [ 'Passwords must be at least six characters long.' ] })

	if passwd != pwconfirm:
		return (False, { 'errors': [ 'The passwords given do not match' ]})

	if ":" in passwd:
		return (False, { 'errors': [ 'The passwords cannot contain semicolons' ]})

	if " " in passwd:
		return (False, { 'errors': [ 'The passwords cannot contain spaces' ]})

	user_props = {
		'username': user,
		'password': passwd,
		'confirm': passwd,
		'roles': [ 'Member' ],
		'domains': [],
		'email': '%s@conga.example.com' % user
	}

	try:
		self.portal_registration.addMember(user, passwd, properties=user_props)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VAU0: %s: %r %s' % (user, e, str(e)))
		return (False, { 'errors': [ 'Unable to add new user "%s": %s ' % (user, str(e)) ] })

	if not self.portal_membership.getMemberById(user):
		return (False, { 'errors': [ 'Unable to add new user "%s"' % user ] })

	messages.append('Added new user "%s" successfully' % user)
	return (True, { 'messages': messages, 'params': { 'user': user }})

def validateAddClusterInitial(self, request):
	errors = list()
	messages = list()

	try:
		request.SESSION.delete('add_cluster_initial')
		request.SESSION.delete('add_cluster')
	except:
		pass

	cur_host = None
	try:
		sysData = request.form['__SYSTEM0']
		if not sysData or len(sysData) < 1:
			raise Exception, 'no node was given'
		cur_host = sysData[0]
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('vACI0: %r %s' % (e, str(e)))
		return (False, { 'errors': [ 'You must provide the address of at least one node in the cluster you wish to add.' ]})

	cur_entry = { 'host': cur_host }
	try:
		if len(sysData) < 2 or not sysData[1]:
			raise Exception, 'no password'
		cur_pass = sysData[1]
		cur_entry['passwd'] = cur_pass
	except:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('vACI1: %s no password given' % cur_host)
		request.SESSION.set('add_cluster_initial', cur_entry)
		return (False, { 'errors': [ 'No password was given for %s' % cur_host ] })

	check_certs = False
	try:
		check_certs = request.form.has_key('check_certs')
	except:
		check_certs = False

	cur_host_trusted = False
	try:
		cur_host_trusted = request.form.has_key('host_is_trusted')
	except:
		cur_host_trusted = not check_certs

	trust_shown = False
	try:
		trust_shown = request.form.has_key('trust_shown')
	except:
		trust_shown = False

	cur_host_fp = None
	try:
		cur_host_fp = request.form['host_fingerprint'].strip()
		if not cur_host_fp:
			cur_host_fp = None
	except:
		cur_host_fp = None

	try:
		rc = RicciCommunicator(cur_host, enforce_trust=False)
		if not rc:
			raise Exception, 'rc is None'
		cur_fp = rc.fingerprint()
		if cur_host_fp is not None:
			cur_entry['fp'] = cur_host_fp
		else:
			cur_entry['fp'] = cur_fp[1]
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('vACI2: %s: %r %s' % (cur_host, e, str(e)))
		request.SESSION.set('add_cluster_initial', cur_entry)
		return (False, { 'errors': [ 'Unable to establish a connection to the ricci agent on %s: %s' % (cur_host, str(e)) ] })

	if not check_certs or cur_host_trusted:
		try:
			if cur_host_fp is not None and cur_host_fp != cur_fp[1]:
				errmsg = 'The key fingerprint for %s has changed from under us. It was "%s" and is now "%s."' \
					% (cur_host, cur_host_fp, cur_fp[1])
				request.SESSION.set('add_cluster_initial', cur_entry)
				luci_log.info('SECURITY: %s' % errmsg)
				return (False, { 'errors': [ errmsg ] })
			rc.trust()
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('vACI3: %s %r %s' \
					% (cur_host, e, str(e)))
			request.SESSION.set('add_cluster_initial', cur_entry)
			return (False, { 'errors': [ 'Unable to establish trust for host %s' % (cur_host, str(e)) ] })
	elif check_certs:
		if not rc.trusted():
			msg = '%s has %s fingerprint %s' \
				% (cur_host, cur_fp[0], cur_fp[1])
		else:
			cur_host_trusted = True
			cur_entry['trusted'] = True
			msg = 'Host %s %s fingerprint %s is already trusted.' \
				% (cur_host, cur_fp[0], cur_fp[1])
		request.SESSION.set('add_cluster_initial', cur_entry)
		messages.append(msg)
		return (True, { 'messages': [ msg ] })

	try:
		request.SESSION.delete('add_cluster_initial')
	except:
		pass

	rc = None
	try:
		rc = RicciCommunicator(cur_host, enforce_trust=True)
		if not rc:
			raise Exception, 'rc is None'
		cur_entry['trusted'] = rc.trusted()
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('vACI4: %s %r %s' % (cur_host, e, str(e)))
		request.SESSION.set('add_cluster_initial', cur_entry)
		return (False, { 'errors': [ 'Unable to connect to the ricci agent on %s' % cur_host ] })

	prev_auth = rc.authed()
	if not prev_auth:
		try:
			rc.auth(cur_pass)
			if not rc.authed():
				raise Exception, 'authentication failed'
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('vACI5: %s: %r %s' \
					% (cur_host, e, str(e)))
			request.SESSION.set('add_cluster_initial', cur_entry)
			return (False, { 'errors': [ 'Unable to authenticate to the ricci agent on "%s"' % cur_host ] })

	del cur_entry

	try:
		cluster_info = rc.cluster_info()
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('vACI5aa: %s: %r %s' % (cur_host, e, str(e)))
		cluster_info = None

	if not cluster_info or not cluster_info[0]:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('vACI5a0: no cluster info')

		if not prev_auth:
			try:
				rc.unauth()
			except:
				pass

		if not cluster_info:
			errmsg = 'An error occurred while attempting to retrieve the cluster.conf file from "%s"' % cur_host
		else:
			errmsg = '"%s" reports is not a member of any cluster.' % cur_host
		return (False, { 'errors': [ errmsg ] })

	cluster_name = cluster_info[0]

	# Make sure a cluster with this name is not already managed before
	# going any further.
	try:
		dummy = self.restrictedTraverse('%s%s' \
					% (CLUSTER_FOLDER_PATH, cluster_name))
		if not dummy:
			raise Exception, 'no existing cluster'
		errors.append('A cluster named "%s" is already managed.')
		if not prev_auth:
			try:
				rc.unauth()
			except:
				pass
		return (False, { 'errors': errors })
	except:
		pass

	cluster_os = resolveOSType(rc.os())
	if LUCI_DEBUG_MODE is True:
		luci_log.debug_verbose('vACI5a: cluster os is %s (%s)' \
			% (cluster_os, rc.os()))

	try:
		cluster_conf = getClusterConf(rc)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('vACI5a1: no cluster.conf: %r %s' \
				% (e, str(e)))
		cluster_conf = None

	if cluster_conf:
		try:
			node_list = getClusterConfNodes(cluster_conf)
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('vACI5b: no node list: %r %s' \
					% (e, str(e)))
			node_list = None

	if not cluster_conf or not node_list or len(node_list) < 1:
		if not prev_auth:
			try:
				rc.unauth()
			except:
				pass
		return (False, { 'errors': [ 'Error retrieving the nodes list for cluster "%s" from node "%s"' % (cluster_name, cur_host) ] })

	same_node_passwds = False
	try:
		same_node_passwds = request.form.has_key('allSameCheckBox')
	except:
		same_node_passwds = False

	add_cluster = {
		'name': cluster_name,
		'nodes': {},
		'cluster_os': cluster_os,
		'pass': 0,
		'identical_passwds': same_node_passwds,
		'check_certs': check_certs
	}

	for i in node_list:
		cur_node = { 'host': i }
		if same_node_passwds is True:
			cur_node['passwd'] = cur_pass
		add_cluster['nodes'][i] = cur_node

	request.SESSION.set('add_cluster', add_cluster)
	request.response.redirect('/luci/homebase/index_html?pagetype=%s' \
		% HOMEBASE_ADD_CLUSTER)

def parseHostForm(request, check_certs):
	errors = list()
	messages = list()
	system_list = {}

	try:
		num_storage = int(request.form['numStorage'].strip())
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('PHF1: numStorage field missing: %r %s' \
				% (e, str(e)))
		errors.append('The number of systems entered could not be determined.')

	trust_shown = False
	try:
		trust_shown = request.form.has_key('trust_shown')
	except:
		trust_shown = False

	incomplete = False
	i = 0
	while i < num_storage:
		try:
			sysData = request.form['__SYSTEM%d' % i]
			if len(sysData) < 1 or not sysData[0]:
				raise Exception, 'no hostname'
			cur_host = sysData[0]
			if cur_host in system_list:
				errors.append('You have added "%s" more than once.' % cur_host)
				raise Exception, '%s added more than once' % cur_host
		except:
			i += 1
			continue

		cur_system = { 'host': cur_host }

		if len(sysData) < 2 or not sysData[1]:
			errors.append('No password for %s (entry %d).' % (cur_host, i))
			cur_passwd = None
		else:
			cur_passwd = sysData[1]
			cur_system['passwd'] = cur_passwd

		try:
			cur_fp = request.form['__SYSTEM%dFingerprint' % i].strip()
			if not cur_fp:
				raise Exception, 'fingerprint is blank'
			cur_system['fp'] = cur_fp
		except:
			cur_fp = None

		try:
			cur_set_trust = request.form.has_key('__SYSTEM%dTrusted' % i)
		except:
			cur_set_trust = False

		if check_certs or (cur_fp is not None and cur_set_trust is True):
			try:
				rc = RicciCommunicator(cur_host, enforce_trust=False)
				if not rc:
					raise Exception, 'rc is None'
				cur_system['prev_auth'] = rc.authed()
				fp = rc.fingerprint()

				if cur_set_trust is True and cur_fp is not None:
					cur_system['fp'] = cur_fp
					if cur_fp != fp[1]:
						errmsg = 'The key fingerprint for %s has changed from under us. It was "%s" and is now "%s."' % (cur_host, cur_fp, fp[1])
						errors.append(errmsg)
						luci_log.info('SECURITY: %s' % errmsg)
						cur_system['error'] = True
						incomplete = True
					else:
						rc.trust()
						cur_system['trusted'] = True
				else:
					cur_system['fp'] = fp[1]

				if not rc.trusted():
					incomplete = True
					msg = '%s has %s fingerprint %s' % (cur_host, fp[0], fp[1])
				else:
					cur_system['trusted'] = True
					msg = '%s %s fingerprint %s is already trusted.' % (cur_host, fp[0], fp[1])

				if check_certs:
					messages.append(msg)
			except Exception, e:
				cur_system['error'] = True
				try:
					del cur_system['trusted']
				except:
					pass
				errors.append('Unable to retrieve the SSL fingerprint for node %s: %s' % (cur_host, str(e)))
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('PHF2: %s: %r %s' \
						% (cur_host, e, str(e)))
		else:
			try:
				rc = RicciCommunicator(cur_host, enforce_trust=False)
				if not rc:
					raise Exception, 'rc is None'

				rc.trust()
				cur_system['trusted'] = True
				cur_system['prev_auth'] = rc.authed()
			except Exception, e:
				incomplete = True
				cur_system['error'] = True
				try:
					if not 'prev_auth' in cur_system:
						del cur_system['trusted']
						rc.untrust()
				except:
					pass
				errors.append('Unable to add the key for node %s to the trusted keys list.' % cur_host)
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('PHF3: %s: %r %s' \
						% (cur_host, e, str(e)))
		system_list[cur_host] = cur_system
		i += 1

	return system_list, incomplete, errors, messages

def validateAddCluster(self, request):
	errors = list()

	try:
		request.SESSION.delete('add_cluster')
		request.SESSION.delete('add_cluster_initial')
	except:
		pass

	cluster_name = '[unknown]'
	try:
		cluster_name = request.form['clustername'].strip()
	except:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VAC0: no cluster name')
		errors.append('No cluster name was given.')

	cluster_os = '[unknown]'
	try:
		cluster_os = request.form['cluster_os'].strip()
	except:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VAC1: no cluster os')
		errors.append('Unable to determine the version of cluster %s.' % cluster_name)

	check_certs = request.form.has_key('check_certs')
	same_node_passwds = request.form.has_key('allSameCheckBox')

	try:
		pass_num = int(request.form['pass'].strip()) + 1
	except:
		pass_num = 1

	add_cluster = {
		'name': cluster_name,
		'pass': pass_num,
		'cluster_os': cluster_os,
		'identical_passwds': same_node_passwds,
		'check_certs': check_certs
	}

	system_list, incomplete, new_errors, messages = parseHostForm(request, check_certs)
	errors.extend(new_errors)
	add_cluster['nodes'] = system_list

	for i in system_list:
		cur_system = system_list[i]

		cur_host_trusted = 'trusted' in cur_system
		cur_host = cur_system['host']
		prev_auth = False
		try:
			cur_passwd = cur_system['passwd']
		except:
			cur_passwd = None

		if (cur_host_trusted or not check_certs) and cur_passwd:
			try:
				rc = RicciCommunicator(cur_host)
				prev_auth = rc.authed()
			except Exception, e:
				errors.append('Unable to connect to the ricci agent on %s: %s' \
					% (cur_host, str(e)))
				incomplete = True
				cur_system['errors'] = True
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('VAC2: %s: %r %s' \
						% (cur_host, e, str(e)))
				continue

			try:
				rc.auth(cur_passwd)
				if not rc.authed():
					raise Exception, 'authentication failed'
			except Exception, e:
				errors.append('Unable to authenticate to the ricci agent on %s: %s' \
					% (cur_host, str(e)))
				incomplete = True
				cur_system['errors'] = True
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('VAC3: %s: %r %s' \
						% (cur_host, e, str(e)))
				continue

			cluster_info = rc.cluster_info()
			if cluster_info[0] != cluster_name and cluster_info[1] != cluster_name:
				incomplete = True
				cur_system['errors'] = True

				if cluster_info[0]:
					cur_cluster_name = cluster_info[0]
				else:
					cur_cluster_name = cluster_info[1]

				if cur_cluster_name:
					err_msg = 'Node %s reports it is in cluster "%s" and we expect "%s"' \
						% (cur_host, cur_cluster_name, cluster_name)
				else:
					err_msg = 'Node %s reports it is not a member of any cluster' % cur_host

				if not prev_auth:
					try:
						rc.unauth()
					except Exception, e:
						if LUCI_DEBUG_MODE is True:
							luci_log.debug_verbose('VAC4: %s: %r %s' \
								% (cur_host, e, str(e)))

				errors.append(err_msg)
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('VAC5: %s' % err_msg)
				continue

			#cur_os = resolveOSType(rc.os())
			#if cur_os != cluster_os:
			#	if LUCI_DEBUG_MODE is True:
			#		luci_log.debug_verbose('VAC5a: "%s" / "%s" -> "%s"' \
			#			% (cluster_os, rc.os(), cur_os))
			#	incomplete = True
			#	cur_system['errors'] = True

			#	if not prev_auth:
			#		try:
			#			rc.unauth()
			#		except Exception, e:
			#			if LUCI_DEBUG_MODE is True:
			#				luci_log.debug_verbose('VAC6: %s: %r %s' \
			#					% (cur_host, e, str(e)))

			#	err_msg = 'Node %s reports its cluster version is %s and we expect %s' \
			#		% (cur_host, cur_os, cluster_os)

			#	errors.append(err_msg)
			#	if LUCI_DEBUG_MODE is True:
			#		luci_log.debug_verbose('VAC7: %s' % err_msg)
			#	continue
		else:
			incomplete = True

	if len(errors) > 0:
		incomplete = True

	if not incomplete or request.form.has_key('asis'):
		err_msg = manageCluster(self, cluster_name, system_list, cluster_os)
		if err_msg:
			incomplete = True
			errors.append('An error occurred while creating the database objects for cluster %s: %s' % (cluster_name, err_msg))
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('VAC7: error adding cluster DB objects for %s: %s' % (cluster_name, err_msg))
		else:
			messages.append('Cluster %s is now managed by Luci' % cluster_name)
			incomplete = False

	if incomplete:
		add_cluster['incomplete'] = True
		request.SESSION.set('add_cluster', add_cluster)
		return_code = False
	else:
		return_code = True

	return (return_code, {'errors': errors, 'messages': messages })

def validateAddSystem(self, request):
	try:
		request.SESSION.delete('add_systems')
	except:
		pass

	check_certs = request.form.has_key('check_certs')

	add_systems, incomplete, errors, messages = parseHostForm(request, check_certs)
	delete_keys = list()
	for i in add_systems:
		cur_system = add_systems[i]

		cur_host_trusted = 'trusted' in cur_system
		cur_host = cur_system['host']

		try:
			cur_passwd = cur_system['passwd']
		except:
			cur_passwd = None

		if (cur_host_trusted or not check_certs) and cur_passwd:
			csResult = createSystem(self, cur_host, cur_passwd)
			if csResult:
				incomplete = True
				cur_system['error'] = True
				errors.append(csResult)
			else:
				delete_keys.append(i)
				messages.append('Added storage system "%s" successfully' \
					% cur_host)

	for i in delete_keys:
		try:
			del add_systems[i]
		except:
			pass

	if len(errors) > 0:
		return_code = False
	else:
		return_code = True

	if incomplete:
		try:
			request.SESSION.set('add_systems', add_systems)
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('validateSA2: %r %s' % (e, str(e)))
		return_code = False
	else:
		try:
			request.SESSION.delete('add_systems')
			del add_systems
		except:
			pass

	return (return_code, { 'errors': errors, 'messages': messages})

def validatePerms(self, request):
	messages = list()
	errors = list()

	username = None
	if request.form.has_key('userList'):
		username = request.form['userList'].strip()

	if not username:
		return (False, { 'errors': [ 'No user name was given' ]})

	user_id = None
	try:
		user = self.portal_membership.getMemberById(username)
		if not user:
			raise Exception, 'no user'
		user_id = user.getUserId()
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VP1: no user "%s": %r %s' \
				% (username, e, str(e)))
		errors.append('An invalid user "%s" was given' % username)

	if len(errors) > 0:
		return (False, { 'errors': errors })

	new_passwd = request.form.get('newpasswd')
	if new_passwd:
		new_passwd = new_passwd.strip()
	if new_passwd:
		try:
			user.userSetPassword(user_id, new_passwd)
			messages.append('Set password for user "%s"' % username)
		except Exception, e:
			errors.append('An error occurred while setting the password for user "%s"' % username)
	if len(errors) > 0:
		return (False, { 'errors': errors })

	clusters = self.restrictedTraverse('%s/systems/cluster/objectItems' % PLONE_ROOT)('Folder')
	if not request.form.has_key('__CLUSTER'):
		for i in clusters:
			try:
				if user.has_role('View', i[1]):
					roles = list(i[1].get_local_roles_for_userid(user_id))
					roles.remove('View')

					if roles:
						i[1].manage_setLocalRoles(user_id, roles)
					else:
						i[1].manage_delLocalRoles([ user_id ])
					messages.append('Removed permission for user "%s" for cluster "%s"' % (user_id, i[0]))
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('VP2: %s %s: %r %s' \
						% (user_id, i[0], e, str(e)))
				errors.append('Failed to remove permission for user "%s" for cluster "%s"' % (user_id, i[0]))
	else:
		for i in clusters:
			if i[0] in request.form['__CLUSTER']:
				try:
					if not user.has_role('View', i[1]):
						roles = list(i[1].get_local_roles_for_userid(user_id))
						roles.append('View')
						i[1].manage_setLocalRoles(user_id, roles)
						messages.append('Added permission for user "%s" for cluster "%s"' % (user_id, i[0]))
				except Exception, e:
					if LUCI_DEBUG_MODE is True:
						luci_log.debug_verbose('VP3: %s %s %r %s' \
							% (user_id, i[0], e, str(e)))
					errors.append('Failed to add permission for user "%s" for cluster "%s"' % (user_id, i[0]))
			else:
				try:
					if user.has_role('View', i[1]):
						roles = list(i[1].get_local_roles_for_userid(user_id))
						roles.remove('View')

						if roles:
							i[1].manage_setLocalRoles(user_id, roles)
						else:
							i[1].manage_delLocalRoles([ user_id ])

						messages.append('Removed permission for user "%s" for cluster "%s"' % (user_id, i[0]))
				except Exception, e:
					if LUCI_DEBUG_MODE is True:
						luci_log.debug_verbose('VP4: %s %s %r %s' \
							% (user_id, i[0], e, str(e)))
					errors.append('Failed to remove permission for user "%s" for cluster "%s"' % (user_id, i[0]))


	storage = self.restrictedTraverse('%s/systems/storage/objectItems' % PLONE_ROOT)('Folder')
	if not request.form.has_key('__SYSTEM'):
		for i in storage:
			try:
				if user.has_role('View', i[1]):
					roles = list(i[1].get_local_roles_for_userid(user_id))
					roles.remove('View')

					if roles:
						i[1].manage_setLocalRoles(user_id, roles)
					else:
						i[1].manage_delLocalRoles([ user_id ])
					messages.append('Removed permission for user "%s" for system "%s"' % (user_id, i[0]))
			except:
				errors.append('Failed to remove permission for user "%s" for system "%s"' % (user_id, i[0]))
	else:
		for i in storage:
			if i[0] in request.form['__SYSTEM']:
				try:
					if not user.has_role('View', i[1]):
						roles = list(i[1].get_local_roles_for_userid(user_id))
						roles.append('View')
						i[1].manage_setLocalRoles(user_id, roles)
						messages.append('Added permission for user "%s" for system "%s"' % (user_id, i[0]))
				except:
					errors.append('Failed to add permission for user "%s" for system "%s"' % (user_id, i[0]))
			else:
				try:
					if user.has_role('View', i[1]):
						roles = list(i[1].get_local_roles_for_userid(user_id))
						roles.remove('View')

						if roles:
							i[1].manage_setLocalRoles(user_id, roles)
						else:
							i[1].manage_delLocalRoles([ user_id ])

						messages.append('Removed permission for user "%s" for system "%s"' % (user_id, i[0]))
				except:
					errors.append('Failed to remove permission for user "%s" for system "%s"' % (user_id, i[0]))

	if len(errors) > 0:
		ret = False
	else:
		ret = True

	return (ret , {'errors': errors, 'messages': messages, 'params': {'user': user_id }})

def validateAuthenticate(self, request):
	try:
		request.SESSION.delete('auth_systems')
		request.SESSION.delete('auth_status')
	except:
		pass

	check_certs = request.form.has_key('check_certs')

	system_list, incomplete, errors, messages = parseHostForm(request, check_certs)
	delete_keys = list()
	for i in system_list:
		cur_system = system_list[i]

		cur_host_trusted = 'trusted' in cur_system
		cur_host = cur_system['host']

		try:
			cur_passwd = cur_system['passwd']
		except:
			cur_passwd = None

		if (cur_host_trusted or not check_certs) and cur_passwd:
			try:
				rc = RicciCommunicator(cur_host, enforce_trust=True)
				if not rc:
					raise Exception, 'connection failed'
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('validateAuth0: %s: %r %s' \
						% (cur_host, e, str(e)))
				errors.append('Unable to communicate with the ricci agent on %s: %s' \
					% (cur_host, str(e)))
				incomplete = True
				cur_system['error'] = True
				continue

			try:
				if rc.authed():
					messages.append('%s is already authenticated.' % cur_host)
				else:
					rc.auth(cur_passwd)
					if not rc.authed():
						raise Exception, 'authentication failed'
					messages.append('Authenticated to %s successfully' \
						% cur_host)
				delete_keys.append(i)
				try:
					clearNodeStatus(self, getStorageNode(self, cur_host), CLUSTER_NODE_NEED_AUTH)
				except:
					pass

				try:
					clearNodeStatus(self, getClusterNode(self, cur_host, rc.cluster_info()[0]), CLUSTER_NODE_NEED_AUTH)
				except:
					pass
			except Exception, e:
				errors.append('Unable to authenticate to %s: %s' % (cur_host, str(e)))
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('validateAuth1: %s: %r %s' \
						% (cur_host, e, str(e)))
				incomplete = True
				cur_system['error'] = True

	for i in delete_keys:
		try:
			del system_list[i]
		except:
			pass

	if incomplete:
		try:
			request.SESSION.set('auth_systems', system_list)
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('validateAuthenticate2: %r' \
					% (e, str(e)))
	else:
		try:
			request.SESSION.delete('auth_systems')
		except:
			pass

	auth_msgs = {}
	if len(errors) > 0:
		auth_msgs['errors'] = errors
	if len(messages) > 0:
		auth_msgs['messages'] = messages
	request.SESSION.set('auth_status', auth_msgs)
	request.response.redirect('/luci/homebase/index_html?pagetype=5')

formValidators = [
	validateAddUser,
	validateAddSystem,
	validatePerms,
	validateDelUser,
	validateDelSystem,
	validateAddCluster,
	validateAddClusterInitial,
	validateAuthenticate
]

def homebaseControlPost(self, request):
	if 'ACTUAL_URL' in request:
		url = request['ACTUAL_URL']
	else:
		url = '.'

	if 'pagetype' in request.form:
		pagetype = int(request.form['pagetype'])
	else:
		try:
			request.SESSION.set('checkRet', {})
		except:
			pass
		return homebasePortal(self, '.', '0')

	try:
		validatorFn = formValidators[pagetype - 1]
	except:
		try:
			request.SESSION.set('checkRet', {})
		except:
			pass
		return homebasePortal(self, '.', '0')

	ret = validatorFn(self, request)
	params = None

	if ret and len(ret) > 1 and ret[1]:
		if 'params' in ret[1]:
			params = ret[1]['params']
		request.SESSION.set('checkRet', ret[1])

	return homebasePortal(self, url, pagetype, params)

def homebaseControl(self, request):
	if request.REQUEST_METHOD == 'POST':
		return homebaseControlPost(self, request)

	try:
		request.SESSION.set('checkRet', {})
	except:
		pass

	if request.has_key('ACTUAL_URL'):
		url = request['ACTUAL_URL']
	else:
		url = '.'

	if request.has_key('pagetype'):
		pagetype = request['pagetype']
	else:
		pagetype = '0'

	return homebasePortal(self, url, pagetype)

def homebasePortal(self, url=None, pagetype=None, params=None):
	ret = {}
	temp = list()
	index = 0
	cur = None

	if pagetype is None:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('HBP0: pagetype is None')

	pagetype = str(pagetype)

	# The Add System page
	try:
		if havePermAddStorage(self):
			addSystem = {}
			addSystem['Title'] = 'Add a System'
			addSystem['absolute_url'] = '%s?pagetype=%s' \
				% (url, HOMEBASE_ADD_SYSTEM)
			addSystem['Description'] = 'Add a system to the Luci storage management interface.'

			if pagetype == HOMEBASE_ADD_SYSTEM:
				addSystem['currentItem'] = True
				ret['curIndex'] = index
				cur = addSystem
			index += 1
			temp.append(addSystem)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('HBP1: %r %s' % (e, str(e)))

	# Initial add cluster page
	try:
		if havePermAddCluster(self):
			addCluster = {}
			addCluster['Title'] = 'Add an Existing Cluster'
			addCluster['absolute_url'] = '%s?pagetype=%s' \
				% (url, HOMEBASE_ADD_CLUSTER_INITIAL)
			addCluster['Description'] = 'Add an existing cluster to the Luci cluster management interface.'

			if pagetype == HOMEBASE_ADD_CLUSTER_INITIAL or pagetype == HOMEBASE_ADD_CLUSTER:
				addCluster['currentItem'] = True
				ret['curIndex'] = index
				cur = addCluster
			index += 1
			temp.append(addCluster)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('HBP2: %r %s' % (e, str(e)))

	# The Remove Storage System page
	try:
		if (havePermRemStorage(self) and havePermRemCluster(self) and (getStorage(self) or getClusters(self))):
			remSystem = {}
			remSystem['Title'] = 'Manage Systems'
			remSystem['absolute_url'] = '%s?pagetype=%s' \
				% (url, HOMEBASE_DEL_SYSTEM)
			remSystem['Description'] = 'Update or remove storage systems and clusters.'
			if pagetype == HOMEBASE_DEL_SYSTEM:
				remSystem['currentItem'] = True
				ret['curIndex'] = index
				cur = remSystem
			index += 1
			temp.append(remSystem)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('HBP3: %r %s' % (e, str(e)))

	# Add a Luci user.
	try:
		if havePermAddUser(self):
			addUser = {}
			addUser['Title'] = 'Add a User'
			addUser['absolute_url'] = '%s?pagetype=%s' \
				% (url, HOMEBASE_ADD_USER)
			addUser['Description'] = 'Add a user to the Luci interface.'
			if pagetype == HOMEBASE_ADD_USER:
				addUser['currentItem'] = True
				ret['curIndex'] = index
				cur = addUser
			index += 1
			temp.append(addUser)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('HBP4: %r %s' % (e, str(e)))

	# Delete a Luci user
	try:
		if (self.portal_membership.listMembers() and havePermDelUser(self)):
			delUser = {}
			delUser['Title'] = 'Delete a User'
			delUser['absolute_url'] = '%s?pagetype=%s' \
				% (url, HOMEBASE_DEL_USER)
			delUser['Description'] = 'Delete a Luci user.'
			if pagetype == HOMEBASE_DEL_USER:
				delUser['currentItem'] = True
				ret['curIndex'] = index
				cur = delUser
			index += 1
			temp.append(delUser)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('HBP5: %r %s' % (e, str(e)))

	# The User Permissions Page
	try:
		if (havePermEditPerms(self) and self.portal_membership.listMembers() and (getStorage(self) or getClusters(self))):
			userPerm = {}
			userPerm['Title'] = 'User Permissions'
			userPerm['absolute_url'] = '%s?pagetype=%s' % (url, HOMEBASE_PERMS)
			userPerm['Description'] = 'Set permissions for Luci users.'
			if pagetype == HOMEBASE_PERMS:
				userPerm['currentItem'] = True
				ret['curIndex'] = index
				cur = userPerm
			index += 1
			temp.append(userPerm)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('HBP6: %r %s' % (e, str(e)))

	if not ret.has_key('curIndex'):
		ret['curIndex'] = 0

	if cur is not None and cur.has_key('absolute_url'):
		cur['base_url'] = cur['absolute_url']

		if params:
			import cgi
			param_list = list()
			for i in params:
				param_list.append('&%s=%s' \
					% (cgi.escape(i), cgi.escape(params[i])))
			temp = '%s%s' % (cur['absolute_url'], ''.join(param_list))
			cur['absolute_url'] = temp

	ret['children'] = temp
	return ret

def createSystem(self, host, passwd):
	try:
		dummy = self.restrictedTraverse('%s%s' \
			% (STORAGE_FOLDER_PATH, host)).objectItems()
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CS0: %s already exists' % host)
		return 'Storage system %s is already managed' % host
	except:
		pass

	try:
		rc = RicciCommunicator(host, enforce_trust=True)
		if rc is None:
			raise Exception, 'rc is None'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CS2: %s: %r %s' % (host, e, str(e)))
		return 'Unable to establish a secure connection to the ricci agent on %s: %s' % (host, str(e))

	try:
		if not rc.authed():
			rc.auth(passwd)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CS3: %s: %r %s' % (host, e, str(e)))
		return 'Unable to communicate with the ricci agent on %s for authentication' % host

	try:
		i = rc.authed()
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CS4 %s: %r %s' % (host, e, str(e)))
		return 'Unable to authenticate to the ricci agent on %s' % host

	if i is not True:
		return 'Authentication for storage system %s failed' % host

	try:
		dummy = self.restrictedTraverse('%s%s' % (STORAGE_FOLDER_PATH, host)).objectItems()
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CS4 %s already exists' % host)
		return 'Storage system %s is already managed' % host
	except:
		pass

	try:
		ssystem = self.restrictedTraverse(STORAGE_FOLDER_PATH)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CS5 %s: %r %s' % (host, e, str(e)))
		return 'Unable to create storage system %s: %s' % host

	try:
		ssystem.manage_addFolder(host, '__luci__:system')
		newSystem = self.restrictedTraverse('%s%s' % (STORAGE_FOLDER_PATH, host))
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CS6 %s: %r %s' % (host, e, str(e)))
		return 'Unable to create DB entry for storage system %s' % host

	try:
		newSystem.manage_acquiredPermissions([])
		newSystem.manage_role('View', ['Access contents information', 'View'])
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CS7 %s: %r %s' % (host, e, str(e)))
		return 'Unable to set permissions on storage system %s' % host

	return None

def getDefaultUser(self, request):
	try:
		user = request.form['userList']
	except KeyError, e:
		try:
			user = request['user']
		except:
			try:
				members = map(lambda x: x.getUserName(), self.portal_membership.listMembers())
				members.sort()
				user = members[0]
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('getDefaultUser0: %r %s' \
						% (e, str(e)))
				user = None

	if not user:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('getDefaultUser1: user is none')
	return user

def getUserPerms(self):
	perms = {}

	try:
		members = list()
		members.extend(self.portal_membership.listMembers())
		if len(members) < 1:
			raise Exception, 'no portal members exist'
		members.sort()
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('getUserPerms0: %r %s' % (e, str(e)))
		return {}

	for i in members:
		userName = i.getUserName()

		perms[userName] = {}
		perms[userName]['cluster'] = {}
		perms[userName]['storage'] = {}

		try:
			clusters = self.restrictedTraverse('%s/systems/cluster/objectItems' % PLONE_ROOT)('Folder')
			storage = self.restrictedTraverse('%s/systems/storage/objectItems' % PLONE_ROOT)('Folder')
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('getUserPerms1: user %s: %r %s' \
					% (userName, e, str(e)))
			continue

		for c in clusters:
			try:
				perms[userName]['cluster'][c[0]] = i.has_role('View', c[1])
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('getUserPerms2: user %s, obj %s: %r %s' % (userName, c[0], e, str(e)))

		for s in storage:
			try:
				perms[userName]['storage'][s[0]] = i.has_role('View', s[1])
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('getUserPerms2: user %s, obj %s: %r %s' % (userName, s[0], e, str(e)))
	return perms
